;; Transport
;;

(define (domain transport)
  (:requirements :typing :fluents)
  (:types
        location target locatable - object
        vehicle package - locatable
        )

  (:predicates 
     (road ?l1 ?l2 - location)
     (at ?x - locatable ?y - location)
     (in ?x - package ?y - vehicle)
     (has-petrol-station ?l - location)
     (ready-loading ?v - vehicle)
     )

  (:functions
     (capacity ?v - vehicle)
     (road-length ?l1 ?l2 - location)
     (fuel-demand ?l1 ?l2 - location)
     (fuel-left ?v - vehicle)
     (fuel-max ?v - vehicle)
     (package-size ?p - package)
     )

  (:action drive
    :parameters (?v - vehicle ?l1 ?l2 - location)
    :precondition (and
                   (at ?v ?l1)
                   (road ?l1 ?l2)
                   (>= (fuel-left ?v) (fuel-demand ?l1 ?l2))
                   )
    :effect (and
             (not (at ?v ?l1))
             (at ?v ?l2)
             (decrease (fuel-left ?v) (fuel-demand ?l1 ?l2))
             )
    )

  (:action pick-up
    :parameters (?v - vehicle ?l - location ?p - package)
    :precondition (and
                   (at ?v ?l)
                   (at ?p ?l)
                   (>= (capacity ?v) (package-size ?p))
                   (ready-loading ?v)
                   )
    :effect (and
             (not (at ?p ?l))
             (in ?p ?v)
             (decrease (capacity ?v) (package-size ?p))
             ;; lock vehicle
             (not (ready-loading ?v))
             (ready-loading ?v)
             )
    )

  (:action drop
    :parameters (?v - vehicle ?l - location ?p - package)
    :precondition (and
                   (at ?v ?l)
                   (in ?p ?v)
                   (ready-loading ?v)
                   )
    :effect (and
             (not (in ?p ?v))
             (at ?p ?l)
             (increase (capacity ?v) (package-size ?p))
             (not (ready-loading ?v))
             (ready-loading ?v)
             )
    )

  (:action refuel
    :parameters (?v - vehicle ?l - location)
    :precondition (and
                   (at ?v ?l)
                   (has-petrol-station ?l)
                   )
    :effect (assign (fuel-left ?v) (fuel-max ?v))
    )

)
